#include <cstdio>
#include "CHWCPU.h"

const char*
CHWCPU::GetName() const
{
  // Return name
  return "HW_GenericCPU";
}

bool
CHWCPU::Step()
{
  // Is there any instruction in progress?
  if(InstructionInProgress)
  {
    // Continue with execution
    SwitchMode(FIBER_EXECUTION_MODE);

    // Return
    return true;
  }

  // Guess no instruction want's to execute
  // Let's start with interrupt dispatching in that
  // case
  InterruptDispatchPending();

  // Now switch to execution mode
  SwitchMode(FIBER_EXECUTION_MODE);

  // Done
  return true;
}

CHWCPU::CHWCPU()
{
  // Setup fiber
  FiberHandle[FIBER_NORMAL_MODE] = CS::MainFiber;
  this->ExecutionFinish = false;  
}

bool
CHWCPU::InitCPU()
{
  // Create the execution fiber
  FiberHandle[FIBER_EXECUTION_MODE] = CreateFiber(0, this->ExecutionFiberProcWrapper, this);

  // Switch to execution fiber to let it initialize itself
  SwitchToFiber(FiberHandle[FIBER_EXECUTION_MODE]);

  // Done
  return true;
}

bool
CHWCPU::SwitchMode(FiberMode Mode)
{
  // Basically call SwitchToFiber ;p
  SwitchToFiber(FiberHandle[Mode]);

  // Done
  return true;  
}

bool
CHWCPU::YieldExec()
{
  // This function CANNOT be called in normal mode
  return SwitchMode(FIBER_NORMAL_MODE);
}

VOID
CHWCPU::ExecutionFiberProcWrapper(PVOID lpParameter)
{
  // Just a wrapper...
  ((CHWCPU*)lpParameter)->ExecutionFiberProc();
}

void
CHWCPU::ExecutionFiberProc()
{
  // Initialize something!
  this->Exec_Init();

  // Done for now
  this->YieldExec();

  // Basically, decode and execute ;)))
  while(!this->ExecutionFinish)
  {
    // Decode instruciton
    if(this->Exec_Decode())
    {
      // Execute if decoded
      this->Exec_Execute();
    }

    // Done for now
    this->YieldExec();
  }
}

// Stubs 
bool CHWCPU::Exec_Init()    { return true; }
bool CHWCPU::Exec_Decode()  { return true; }
bool CHWCPU::Exec_Execute() { return true; }

