#include "CHWDebugConsole.h"
#include "CCommon.h"
#include <cstdio>

CHWDebugConsole::CHWDebugConsole()
{
  // Reserve ports and interrupts
  CS::AcquirePort(PORT_DATA, this);
  CS::AcquireInterrupt(INT_FLUSH, this);

  // Set buffer size for the port
  CS::System->Ports[PORT_DATA].SetType(CSPort::PORT_BUFFERED);
  CS::System->Ports[PORT_DATA].SetBufferSize(PORT_DATA_BUFFER_SIZE);
  CS::System->Ports[PORT_DATA].SetInterrupt(INT_FLUSH, PORT_DATA_BUFFER_SIZE);
}

const char *
CHWDebugConsole::GetName() const
{
  return "HW_DebugConsole";
}

bool
CHWDebugConsole::InterruptDispatcher(int IntNo)
{
  // Dispatch
  switch(IntNo)
  {
    case INT_FLUSH:
      this->FlushData();
      break;
      
    default:
      // Bogus things are going on
      return false;
  }

  // Done
  return true;
}

bool
CHWDebugConsole::Step()
{
  // Basically, everything is handled by ports
  // and interrupts, so... nothing to do here :)
  return true;
}

bool
CHWDebugConsole::FlushData()
{
  char Buffer[32] = {0};

  // Get the data from the port to the buffer
  int i;
  for(i = 0; i < PORT_DATA_BUFFER_SIZE; i++)
  {
    // Get byte
    uint8_t Byte;
    if(!CS::System->Ports[PORT_DATA].Recv(&Byte))
      break; // No more bytes

    // Save byte
    Buffer[i] = (char)Byte;
  }

  // Print it :)
  fwrite(Buffer, 1, i, stdout);

  // Done
  return true;
}

