#pragma once
#include "CHWCPU.h"

class CHWMyCPU : public CHWCPU
{
  protected:
    static const int PORT_RECV_RAM = 0x0070;

    uint8_t ROM[0x4000];
    uint16_t Reg[8];

    enum Register
    {
      REG_R0,
      REG_R1,
      REG_R2,
      REG_R3,
      REG_R4,
      REG_R5,
      REG_R6,
      REG_R7,
      REG_PC = REG_R7
    };

    enum Opcode
    {
      OP_MOV = 0x00,
      OP_ADD = 0x01,
      OP_LDI = 0x02,
      OP_LDR = 0x03,
      OP_HLT = 0xFF
    };

    enum ArgType
    {
      ARG_REG,
      ARG_IMM16,
      ARG_ADDR

    };

    union ArgValue
    {
      enum Register Reg;
      uint16_t Imm16;
      uint16_t Addr;
    };

    enum Opcode    CurrentOpcode;
    enum ArgType   CurrentOpcodeArg[3];
    union ArgValue CurrentOpcodeArgValue[3];
    
    bool CPUHalt;

    bool RequestRAMData(void *Buffer, int RecvPort, uint16_t Addr, uint16_t Len);

    // Functions
    bool Exec_Init();
    bool Exec_Decode();
    bool Exec_Execute();

  public:
    CHWMyCPU();
    const char *GetName() const;
    bool LoadROM(const char *FileName);

};

