#include "CHardware.h"

CHardware::~CHardware()
{
  // Do nothing.
}

bool CHardware::InterruptDispatcherWrapper(int IntNo, CHardware *Self)
{
  // Just call the method
  return Self->InterruptDispatcher(IntNo);
}

const char*
CHardware::GetName() const
{
  return "HW_DefaultHardware";
}

CHardwareWithStdInt::InterruptInfo::InterruptInfo()
{
  this->IntPrio        = -1;
  this->IntHandlerFunc = NULL;
}

bool
CHardwareWithStdInt::IntQueueCmp::operator() (std::pair<int,int> a, std::pair<int,int> b)
{
  // Return the interrupt priority comparision result
  return a.second < b.second;
}

CHardwareWithStdInt::CHardwareWithStdInt()
{
  // Init
  this->InterruptPrioInProgress = -1; // No interrupt
}

bool
CHardwareWithStdInt::InterruptDispatcher(int IntNo)
{
  // Check if we have that interrupt registered
  if(this->InterruptMap.find(IntNo) == this->InterruptMap.end())
    return false;

  // Get priority
  int IntPrio = this->InterruptMap[IntNo].IntPrio;

  // Add to queue
  // The interrupt will be dispatched at the next step or later
  // and not sooner
  this->InterruptQueue.insert(std::pair<int,int>(IntNo, IntPrio));

  // Done
  return true;
}

int
CHardwareWithStdInt::InterruptPop()
{
  // Is there anything on the queue at all ?
  if(this->InterruptQueue.size() == 0)
    return -1; // Nope

  // Get the first interrupt number (but do not pop yet)
  std::pair<int,int> IntInfo = *this->InterruptQueue.begin();
  int IntNo   = IntInfo.first;
  int IntPrio = IntInfo.second;
  
  // Does the top-most interrupt have a greater priority
  // (smaller value) than the currently executed one?
  // Is there anything executed at all?
  bool Preempt = false;
  if(this->InterruptPrioInProgress == -1 ||
     this->InterruptPrioInProgress > IntPrio)
    Preempt = true;

  // If we are to Preempt the current interrupt, we must
  // pop the new one, set the prio, and return the value
  // Also, if there was anything currently going on, we
  // must place it on the queue
  if(Preempt)
  {
    // Remove the poped element from the list
    this->InterruptQueue.erase(this->InterruptQueue.begin());

    // Store the old interrupt prio in the queue
    if(this->InterruptPrioInProgress != -1)
      this->InterruptPrioPreempted.push_front(this->InterruptPrioInProgress);

    // Set the current interrupt prio
    this->InterruptPrioInProgress = IntPrio;

    // Return the new interrupt
    return IntNo;
  }

  // No preempt
  return -1;
}

bool 
CHardwareWithStdInt::InterruptFinished()
{
  // Get rid of the current interrupt prio
  this->InterruptPrioInProgress = -1;

  // Was there anything preempted?
  if(this->InterruptPrioPreempted.size() == 0)
  {
    // Nope, return false (because we are no longer in interrupt state)
    return false;
  }

  // Pop the list
  int IntPrio = *this->InterruptPrioPreempted.begin();
  this->InterruptPrioPreempted.pop_front();

  // Set the current interrupt
  this->InterruptPrioInProgress = IntPrio;

  // Return true, we are still in interrupt mode
  return true;
}

bool
CHardwareWithStdInt::InterruptDispatchPending()
{
  // Pop the interrupt
  int IntNo = InterruptPop();

  // Anything to dispatch ?
  if(IntNo == -1)
  {
    // Nope, return false
    return false;
  }

  // Yep, check if there is an interrupt handler for this int
  if(this->InterruptMap.find(IntNo) == this->InterruptMap.end())
  {
    // There is no
    this->InterruptFinished();

    // Return false, since this interrupt was bogus
    return false;
  }

  // Call the dispatcher
  this->InterruptMap[IntNo].IntHandlerFunc(IntNo, this);

  // Dispatched properly
  return true;
}

