#pragma once
#include "CCommon.h"
#include "CLog.h"
#include <map>
#include <set>
#include <list>

// Totally abstract class
class CHardware : public CLog
{
  public:
    static bool InterruptDispatcherWrapper(int IntNo, CHardware *Self);    
    virtual const char *GetName() const;
    virtual bool InterruptDispatcher(int IntNo) = 0;
    virtual bool Step() = 0;
    virtual ~CHardware();
};

// Hardware with standard priority interrupts
class CHardwareWithStdInt : public CHardware
{
  protected:
    struct InterruptInfo
    {
      int IntPrio;
      bool (*IntHandlerFunc)(int IntNo, void *Self);

      InterruptInfo();
    };

    std::map<int,InterruptInfo> InterruptMap;

    // Interrupt queue
    struct IntQueueCmp
    {
      bool operator() (std::pair<int,int> a, std::pair<int,int> b);
    };
    std::multiset<std::pair<int,int>,IntQueueCmp> InterruptQueue;

    // The below variables contain (in order of appearance):
    // - the currently handled interrupt priority
    // - stores the preempted interrupt priorities
    int InterruptPrioInProgress;
    std::list<int> InterruptPrioPreempted;

    // This "pop" is a very complex function - it handles many
    // things including the priority matching according to the
    // InterruptPrioInProgress status, checking the queue 
    // emptiness, and getting the proper interrupt if any is to
    // be used.
    // If the "pop" returns -1 one should continue handling the
    // currently handled interrupt (or continue with the normal
    // code execution if no interrupt is procesed). Otherwise,
    // one should store the state and start executing the new
    // interrupt;
    int InterruptPop();

    // When the code is finished handling the interrupt, it
    // should inform the interrupt manager about it.
    // The InterruptFinished routine will acquire the previously
    // handled interrupt prio from the InterruptPrioPreempted list
    // and will return true if there was a preempter interrupt, or
    // false in case the last interrupt was handled.
    bool InterruptFinished();

    // Dispatch pending
    // This calls the InterruptPop and runs the proper interrupt
    // handler (if it exists; if it doesn't, it calls InterruptFinished)
    // Returns true if anything was dispatched (this doesn't mean
    // that we left the interrupt mode or sth)
    // If InterruptDispatchPending returns true, InterruptFinished 
    // must be called someday :)
    bool InterruptDispatchPending();

  public:
    CHardwareWithStdInt();
    virtual bool InterruptDispatcher(int IntNo);
};


