#include "TGALoader.h"

//constructor
ImageTGA32::ImageTGA32()
{
  // Setup size
  this->Size = 0;

  // Reset header
  memset(&this->Header, 0, sizeof(this->Header));
}

bool
ImageTGA32::Load(const char *FileName)
{
  // Open the file
  FILE *f = fopen(FileName, "rb");
  if(!f)
    return false; // Guess we couldn't open the file

  // Read the header
  size_t Ret;
  Ret = fread(&this->Header, 1, sizeof(this->Header), f);
  if(Ret != sizeof(this->Header))
  {
    // Something went wrong
    fclose(f);
    return false;
  }

  // Skip the id_field
  fseek(f, this->Header.id_field, SEEK_CUR);
 
  // Calculate the size
  this->Size = this->Header.width;

  // Mul by height
  if(this->Size * this->Header.height < this->Size)
  {
    // Int overflow - no fix, bail out
    fclose(f);
    return false;
  }
  else
  {
    // It's OK
    this->Size *= this->Header.height;
  }

  // Mul by bpp/8
  if(this->Size * (this->Header.bpp / 8) < this->Size)
  {
    // Int overflow - no fix, bail out
    fclose(f);
    return false;
  }
  else
  {
    // It's OK
    this->Size *= this->Header.bpp / 8;
  }
  
  // Is it sane ?
  // Sane == smaller or equal too 4096 * 4096 * 4 bytes per pixel
  if(this->Size > (4096 * 4096 * 4))
  {
    // Doesn't seem sane to me
    fclose(f);
    return false;
  }

  // Resize the vector
  this->Data.resize(this->Size);
  // TODO: Some 'allocation OK' check needed

  // Let's setup the line reading part
  uint8_t *LinePtr = NULL;
  int LineSize = this->Header.width * (this->Header.bpp / 8);
  int LineDelta = 0;

  if((this->Header.flip & 0x20))
  {
    // No flip
    LineDelta = LineSize; // Must go down
    LinePtr = &this->Data[0];
  }
  else
  {
    // Flip
    LineDelta = -LineSize; // LineDelta must go up
    LinePtr = &this->Data[this->Size + LineDelta];
  }

  // OK, let's read the lines
  unsigned int i;
  for(i = 0; i < this->Header.height; i++, LinePtr += LineDelta)
  {
    // Read
    Ret = fread(LinePtr, 1, LineSize, f);
    if(Ret != (size_t)LineSize)
      break; // Guess we're done reading
  }

  // Switch the ABGR to RGBA
  /*for(i = 0; i < this->Size; i+=4)
  {
    uint32_t Var = *(uint32_t*)&this->Data[i];
    this->Data[i] = (Var >> 24) & 0xff;
    this->Data[i+1] = (Var >> 16) & 0xff;
    this->Data[i+2] = (Var >> 8) & 0xff;
    this->Data[i+3] = Var & 0xff;
  }*/

  // Thats it!
  fclose(f);
  return true;
}

#ifdef TGALOADER_TEST
int
main(void)
{
  ImageTGA32 tga;
  tga.Load("gfx/szyba.tga");

  FILE *f = fopen("gfx/szyba.raw", "wb");
  fwrite(&tga.Data[0], 1, tga.Size, f);
  fclose(f);

  return 0;
}

#endif

