#include <cstdio>
#include "CCommon.h"
#include "CHWDebugConsole.h"
#include "CHWMemory.h"
#include "CHWMyCPU.h"
#include "CHWCRT.h"

#include "Testlogo.h"

void
SendASCIIZToPort(int PortNo, const char *Text)
{
  while(*Text) 
    CS::System->Ports[PortNo].Send((uint8_t)*Text++);
}

void
SendBytesToPort(int PortNo, const char *Data, int Size)
{
  while(Size-->0) 
    CS::System->Ports[PortNo].Send((uint8_t)*Data++);
}

int
main()
{
  Log::MinimumLevel = LOG_VERBOSE;

  CS::Init();


  CS::AddHardware(new CHWDebugConsole);  

  CHWMemory *mem = new CHWMemory;
  CS::AddHardware(mem);

  CHWMyCPU *cpu = new CHWMyCPU;
  CS::AddHardware(cpu);
  cpu->LoadROM("test.bin");

  CHWCRT *crt = new CHWCRT;
  CS::AddHardware(crt);

  // List stuff
  GLOG(LOG_NORMAL, "Hardware:");
  int i;
  for(i = 0; i < (int)CS::System->Hardware.size(); i++)
  {
    GLOG(LOG_NORMAL, "  Hardware: Name=\"%s\"",
        CS::System->Hardware[i]->GetName());
  }

  GLOG(LOG_NORMAL, "Ports:");
 
  for(i = 0; i < PORT_COUNT; i++)
  {
    if(CS::System->Ports[i].IsAssigned())
    {
      GLOG(LOG_NORMAL, "  Port %.4x: BufferSize=%i Owner=\"%s\"", i, 
        CS::System->Ports[i].GetBufferSize(),
        CS::System->Ports[i].GetOwner()->GetName());
    }
  }

  GLOG(LOG_NORMAL, "Interrupts:");
  for(i = 0; i < INTERRUPT_COUNT; i++)
  {
    if(CS::System->Interrupts[i].IsAssigned())
    {
      GLOG(LOG_NORMAL, "  Int %.2x: Owner=\"%s\"", i,
          CS::System->Interrupts[i].GetOwner()->GetName());
    }
  }



  GLOG(LOG_NORMAL, "Start");

  /*
  SendASCIIZToPort(0x800, "Automatic Flush After 32 Chars!\n");

  SendASCIIZToPort(0x800, "Forced flush\n");
  CS::IssueInterrupt(0xf0);

  GLOG(LOG_NORMAL, "Write to memory");

  CS::System->Ports[0xfff].SetType(CSPort::PORT_BUFFERED);
  CS::System->Ports[0xfff].SetBufferSize(32);

  SendASCIIZToPort(0xfff, "Data in RAM\n");
  SendBytesToPort(CHWMemory::PORT_REQUESTS, "\1\x34\x12\x0C\xff\x0f", 6); // WRITE to 1234h 12 bytes from port 0xfff
  
  GLOG(LOG_NORMAL, "Step to make the RAM controler save the data");
  CS::Step();
  CS::Step();
  CS::Step();
  CS::Step();

  GLOG(LOG_NORMAL, "Get data from memory tu console");
  SendBytesToPort(CHWMemory::PORT_REQUESTS, "\0\x34\x12\x0C\0\x08", 6); // READ from 1234h 12 bytes to port 0x800

  GLOG(LOG_NORMAL, "Step to make the RAM controler read the data");
  CS::Step();

  GLOG(LOG_NORMAL, "Issue flush interrupt");
  CS::IssueInterrupt(0xf0);

  CS::Step();
  CS::Step();
  CS::Step();
  CS::Step();
  */

  i = 0;
  int j = 0;

  for(i = 0; i < 2048; i++)
  {
    uint8_t tmp = LogoData[i];
    LogoData[i] = 0;
    LogoData[i] |= (((tmp >> 0) & 1) << 7);
    LogoData[i] |= (((tmp >> 1) & 1) << 6);
    LogoData[i] |= (((tmp >> 2) & 1) << 5);
    LogoData[i] |= (((tmp >> 3) & 1) << 4);
    LogoData[i] |= (((tmp >> 4) & 1) << 3);
    LogoData[i] |= (((tmp >> 5) & 1) << 2);
    LogoData[i] |= (((tmp >> 6) & 1) << 1);
    LogoData[i] |= (((tmp >> 7) & 1) << 0);
  }

  SendBytesToPort(CHWCRT::PORT_MODE, "\x04", 1);

  // Step a little
  for(i = 0; i < 512; i++) CS::Step();

  // Halt at the end of the screen
  SendBytesToPort(CHWCRT::PORT_HALT, "\x01", 1);
  for(i = 0; i < 65544; i++) CS::Step();
  SendBytesToPort(CHWCRT::PORT_HALT, "\x00", 1);

  i = 128;
  while(CS::Step())
  {
    // Feed the data to the mono ports

    if(i == 128)
    {
      int x, y;
      x = j % 16;
      y = j / 16;
      SendBytesToPort(CHWCRT::PORT_IN_DATA_MONO, (const char*)&LogoData[x + (127 - y) * 16], 4);
      i = 0;

      j+=4;

      if(j == 16 * 128)
        j = 0;

    }

    i++;


  }




  return 0;
}

